@@ -701,12 +701,11 @@ TkWmProtocolEventProc(
 	    Tcl_Preserve(protPtr);
 	    interp = protPtr->interp;
 	    Tcl_Preserve(interp);
-	    result = Tcl_EvalEx(interp, protPtr->command, -1, TCL_EVAL_GLOBAL);
+	    result = Tcl_GlobalEval(interp, protPtr->command);
 	    if (result != TCL_OK) {
-		Tcl_AddErrorInfo(interp, "\n    (command for \"");
-		Tcl_AddErrorInfo(interp,
-			Tk_GetAtomName((Tk_Window) winPtr, protocol));
-		Tcl_AddErrorInfo(interp, "\" window manager protocol)");
+		Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
+			"\n    (command for \"%s\" window manager protocol)",
+			Tk_GetAtomName((Tk_Window) winPtr, protocol)));
 		Tcl_BackgroundError(interp);
 	    }
 	    Tcl_Release(interp);
@@ -764,7 +763,10 @@ Tk_MacOSXIsAppInFront(void)
  * Custom content view for Tk NSWindows, containing standard NSView subviews.
  * The goal is to emulate X11-style drawing in response to Expose events:
  * during the normal AppKit drawing cycle, we supress drawing of all subviews
- * and instead send Expose events about the subviews that would be redrawn.
+ * (using a technique adapted from WebKit's WebHTMLView) and instead send
+ * Expose events about the subviews that would be redrawn. Tk Expose event
+ * handling and drawing handlers then draw the subviews manually via their
+ * -displayRectIgnoringOpacity:
  */
 
 @interface TKContentView(TKWindowEvent)
@@ -803,6 +805,15 @@ ExposeRestrictProc(
 	    NSCompositeSourceOver);
 #endif
 
+    NSWindow *w = [self window];
+
+    if ([self isOpaque] && [w showsResizeIndicator]) {
+	NSRect bounds = [self convertRect:[w _growBoxRect] fromView:nil];
+
+	if ([self needsToDrawRect:bounds]) {
+	    NSEraseRect(bounds);
+	}
+    }
 
     CGFloat height = [self bounds].size.height;
     HIMutableShapeRef drawShape = HIShapeCreateMutable();
@@ -935,7 +946,153 @@ ExposeRestrictProc(
 }
 
 @end
+
+#pragma mark TKContentViewPrivate
 
+/*
+ * Technique adapted from WebKit/WebKit/mac/WebView/WebHTMLView.mm to supress
+ * normal AppKit subview drawing and make all drawing go through us.
+ * Overrides NSView internals.
+ */
+
+@interface TKContentView(TKContentViewPrivate)
+- (id) initWithFrame: (NSRect) frame;
+- (void) _setAsideSubviews;
+- (void) _restoreSubviews;
+@end
+
+@interface NSView(TKContentViewPrivate)
+- (void) _recursiveDisplayRectIfNeededIgnoringOpacity: (NSRect) rect
+	isVisibleRect: (BOOL) isVisibleRect
+	rectIsVisibleRectForView: (NSView *) visibleView
+	topView: (BOOL) topView;
+- (void) _recursiveDisplayAllDirtyWithLockFocus: (BOOL) needsLockFocus
+	visRect: (NSRect) visRect;
+- (void) _recursive: (BOOL) recurse
+	displayRectIgnoringOpacity: (NSRect) displayRect
+	inContext: (NSGraphicsContext *) context topView: (BOOL) topView;
+- (void) _lightWeightRecursiveDisplayInRect: (NSRect) visRect;
+- (BOOL) _drawRectIfEmpty;
+- (void) _drawRect: (NSRect) inRect clip: (BOOL) clip;
+- (void) _setDrawsOwnDescendants: (BOOL) drawsOwnDescendants;
+@end
+
+@implementation TKContentView(TKContentViewPrivate)
+
+- (id) initWithFrame: (NSRect) frame
+{
+    self = [super initWithFrame:frame];
+    if (self) {
+	_savedSubviews = nil;
+	_subviewsSetAside = NO;
+	[self _setDrawsOwnDescendants:YES];
+    }
+    return self;
+}
+
+- (void) _setAsideSubviews
+{
+#ifdef TK_MAC_DEBUG
+    if (_subviewsSetAside || _savedSubviews) {
+	Tcl_Panic("TKContentView _setAsideSubviews called incorrectly");
+    }
+#endif
+    _savedSubviews = _subviews;
+    _subviews = nil;
+    _subviewsSetAside = YES;
+}
+
+- (void) _restoreSubviews
+{
+#ifdef TK_MAC_DEBUG
+    if (!_subviewsSetAside || _subviews) {
+	Tcl_Panic("TKContentView _restoreSubviews called incorrectly");
+    }
+#endif
+    _subviews = _savedSubviews;
+    _savedSubviews = nil;
+    _subviewsSetAside = NO;
+}
+
+- (void) _recursiveDisplayRectIfNeededIgnoringOpacity: (NSRect) rect
+	isVisibleRect: (BOOL) isVisibleRect
+	rectIsVisibleRectForView: (NSView *) visibleView
+	topView: (BOOL) topView
+{
+    [self _setAsideSubviews];
+    [super _recursiveDisplayRectIfNeededIgnoringOpacity:rect
+	    isVisibleRect:isVisibleRect rectIsVisibleRectForView:visibleView
+	    topView:topView];
+    [self _restoreSubviews];
+}
+
+- (void) _recursiveDisplayAllDirtyWithLockFocus: (BOOL) needsLockFocus
+	visRect: (NSRect) visRect
+{
+    BOOL needToSetAsideSubviews = !_subviewsSetAside;
+
+    if (needToSetAsideSubviews) {
+        [self _setAsideSubviews];
+    }
+    [super _recursiveDisplayAllDirtyWithLockFocus:needsLockFocus
+	    visRect:visRect];
+    if (needToSetAsideSubviews) {
+        [self _restoreSubviews];
+    }
+}
+
+- (void) _recursive: (BOOL) recurse
+	displayRectIgnoringOpacity: (NSRect) displayRect
+	inContext: (NSGraphicsContext *) context topView: (BOOL) topView
+{
+    [self _setAsideSubviews];
+    [super _recursive:recurse
+	    displayRectIgnoringOpacity:displayRect inContext:context
+	    topView:topView];
+    [self _restoreSubviews];
+}
+
+- (void) _lightWeightRecursiveDisplayInRect: (NSRect) visRect
+{
+    BOOL needToSetAsideSubviews = !_subviewsSetAside;
+
+    if (needToSetAsideSubviews) {
+        [self _setAsideSubviews];
+    }
+    [super _lightWeightRecursiveDisplayInRect:visRect];
+    if (needToSetAsideSubviews) {
+        [self _restoreSubviews];
+    }
+}
+
+- (BOOL) _drawRectIfEmpty
+{
+    /*
+     * Our -drawRect manages subview drawing directly, so it needs to be called
+     * even if the area to be redrawn is completely obscured by subviews.
+     */
+
+    return YES;
+}
+
+- (void) _drawRect: (NSRect) inRect clip: (BOOL) clip
+{
+#ifdef TK_MAC_DEBUG_DRAWING
+    TKLog(@"-[%@(%p) %s%@]", [self class], self, _cmd,
+	    NSStringFromRect(inRect));
+#endif
+    BOOL subviewsWereSetAside = _subviewsSetAside;
+
+    if (subviewsWereSetAside) {
+        [self _restoreSubviews];
+    }
+    [super _drawRect:inRect clip:clip];
+    if (subviewsWereSetAside) {
+        [self _setAsideSubviews];
+    }
+}
+
+@end
 
 /*
  * Local Variables:
